/*
 *  Copyright (c) 2021 Fraunhofer Institute for Software and Systems Engineering
 *
 *  This program and the accompanying materials are made available under the
 *  terms of the Apache License, Version 2.0 which is available at
 *  https://www.apache.org/licenses/LICENSE-2.0
 *
 *  SPDX-License-Identifier: Apache-2.0
 *
 *  Contributors:
 *       Fraunhofer Institute for Software and Systems Engineering - initial API and implementation
 *
 */

package org.eclipse.edc.sample.extension.api;

import org.eclipse.edc.connector.contract.spi.offer.store.ContractDefinitionStore;
import org.eclipse.edc.connector.contract.spi.types.offer.ContractDefinition;
import org.eclipse.edc.connector.dataplane.spi.pipeline.DataTransferExecutorServiceContainer;
import org.eclipse.edc.connector.dataplane.spi.pipeline.PipelineService;
import org.eclipse.edc.connector.policy.spi.PolicyDefinition;
import org.eclipse.edc.connector.policy.spi.store.PolicyDefinitionStore;
import org.eclipse.edc.policy.model.Policy;
import org.eclipse.edc.policy.model.PolicyType;
import org.eclipse.edc.runtime.metamodel.annotation.Inject;
import org.eclipse.edc.spi.asset.AssetIndex;
import org.eclipse.edc.spi.system.ServiceExtension;
import org.eclipse.edc.spi.system.ServiceExtensionContext;
import org.eclipse.edc.spi.types.domain.DataAddress;
import org.eclipse.edc.spi.types.domain.asset.Asset;
import org.fraunhofer.fit.extension.extractor.MetadataExtractor;

import java.nio.file.Path;

import static org.eclipse.edc.spi.query.Criterion.criterion;

public class FileTransferExtension implements ServiceExtension {

    public static final String USE_POLICY = "use-eu";
    private static final String EDC_ASSET_PATH = "edc.samples.transfer.01.asset.path";
    private static final String EDC_ASSET_PID = "edc.samples.transfer.01.asset.pid";
    private static final String EDC_ASSET_LINK = "edc.samples.transfer.01.asset.link";
    private static final String EDC_ASSET_VERSION = "edc.samples.transfer.01.asset.version";
    @Inject
    private ContractDefinitionStore contractStore;
    @Inject
    private AssetIndex assetIndex;
    @Inject
    private PipelineService pipelineService;
    @Inject
    private DataTransferExecutorServiceContainer executorContainer;
    @Inject
    private PolicyDefinitionStore policyStore;
    @Inject
    private MetadataExtractor metadataExtractor;

    @Override
    public void initialize(ServiceExtensionContext context) {
        var monitor = context.getMonitor();

        var sourceFactory = new FileTransferDataSourceFactory();
        pipelineService.registerFactory(sourceFactory);

        var sinkFactory = new FileTransferDataSinkFactory(monitor, executorContainer.getExecutorService(), 5);
        pipelineService.registerFactory(sinkFactory);

        var policy = createPolicy();
        policyStore.create(policy);

        registerDataEntries(context);
        registerContractDefinition(policy.getUid());

        context.getMonitor().info("File Transfer Extension initialized!");
    }

    private PolicyDefinition createPolicy() {
        return PolicyDefinition.Builder.newInstance()
                .id(USE_POLICY)
                .policy(Policy.Builder.newInstance()
                        .type(PolicyType.SET)
                        .build())
                .build();
    }

    private void registerDataEntries(ServiceExtensionContext context) {
        var assetPathSetting = context.getSetting(EDC_ASSET_PATH, "/tmp/provider/test-document.txt");
        var assetpid = context.getSetting(EDC_ASSET_PID, "");
        var assetDataLink = context.getSetting(EDC_ASSET_LINK, "");
        var assetVersion = context.getSetting(EDC_ASSET_VERSION, "v1");
        var assetPath = Path.of(assetPathSetting);

        var dataAddress = DataAddress.Builder.newInstance()
                .property("type", "File")
                .property("path", assetPath.getParent().toString())
                .property("filename", assetPath.getFileName().toString())
                .build();

        var metadata = metadataExtractor.getMetadata(assetPath.toString(), assetpid, assetDataLink, assetVersion);
        var assetId = "test-document";
        var asset = Asset.Builder.newInstance()
                .id(assetId)
                .contentType(metadata.remove("mimeType").toString())
                .name(metadata.remove("name").toString())
                .description(metadata.remove("description").toString())
                .version(metadata.remove("version").toString())
                .properties(metadata)
                .build();

        assetIndex.create(asset, dataAddress);
    }

    private void registerContractDefinition(String uid) {
        var contractDefinition = ContractDefinition.Builder.newInstance()
                .id("1")
                .accessPolicyId(uid)
                .contractPolicyId(uid)
                .assetsSelectorCriterion(criterion(Asset.PROPERTY_ID, "=", "test-document"))
                .build();

        contractStore.save(contractDefinition);
    }
}
